// ;-*-C++-*- *  Time-stamp: "2010-03-05 15:36:14 hmmr"
/*
 * Author: Andrei Zavada (a.zavada@sussex.ac.uk)
 *
 * License: GPL
 *
 * Initial version: 2009-12-09
 */

#include <cstdlib>
#include "hosted-neurons.h"
#include "standalone-synapses.h"
#include "model.h"


using namespace std;
using namespace CNRun;

int
main()
{
      // create and set up the model
	CModel *Model;
	if ( !(Model = new CModel( "test", new CIntegrateRK65(), 0)) ) {
		cerr << "Failed to create a model\n";
		return -1;
	}

	bool use_lower_level_api = rand() > RAND_MAX/2;

	CNeuronHH_d	*hh;
	C_BaseNeuron	*pulse;
	CSynapseMxAB_dd	*synapse;

	const char
		*hh_label = "our hodgkin-huxley one",
		*pulse_label = "our stimulation source",
		*synapse_label = "our synapse";

      // add our three units
	if ( use_lower_level_api ) {
		hh = new CNeuronHH_d( hh_label,
				      0.2, 0.1, 0.3,		// x, y, z coordinates: values don't matter at present
				      Model, 0 | CN_UOWNED);	// meaning, it's our responsibility to delete this unit
		pulse = new COscillatorDotPoisson( pulse_label, 0.1, 0.2, 0.3, Model, 0)); // this one will be destroyed in ~CModel()
		synapse = new CSynapseMxAB_dd( synapse_label, pulse, hh, Model); // 0 is the default
	} else {
		hh = add_neuron_species( "HH", hh_label);	// only need neuron species and label
		pulse = add_neuron_species( "Pulse", pulse_label);
		synapse = add_synapse_species( "AB", pulse_label, hh_label,
					       0.18);		// also need gsyn, for synapses

      // enable_spikelogging_service
	hh -> enable_spikelogging_service();

	hh -> start_listening( CN_ULISTENING_DISK | CN_ULISTENING_1VARONLY);
	pulse -> start_listening( CN_ULISTENING_DISK);
	synapse -> start_listening( CN_ULISTENING_DISK);
//	Model->listen_dt = 0.;

      // set parameters
	if ( use_lower_level_api )
		hh -> param_value("Idc") = 0.05;
	else {
		list<CModel::STagGroupNeuronParmSet> specs = { CModel::STagGroupNeuronParmSet ("hh", true, "Idc", 0.05) };
		Model->process_paramset_static_tags( specs);
	}

	// specially, set gsyn
	synapse -> set_g_on_target(*hh) = 0.18;

	Model->dump_units();

      // go
	Model->advance( 100);


	delete Model;

	delete hh;

	return 0;
}


// EOF
